@@ -1,12 +1,14 @@
-#' Data assignment
+#' Data assignment (table or expression result)
 #'
 #' Assign a table or an expression result to a R symbol in the Datashield R session.
-#'
+#' Note that usage of usage of respectively \code{\link{datashield.assign.table}} or 
+#' \code{\link{datashield.assign.expr}} should be preferred for readability.
+#' 
 #' @param conns \code{\link{DSConnection-class}} object or a list of \code{\link{DSConnection-class}}s.
 #' @param symbol Name of the R symbol.
 #' @param value Fully qualified name of a table reference in data repositories (see
-#'   \code{\link{datashield.assign.table}} for more details) or a R expression with allowed assign
-#'  functions calls.
+#'   \code{\link{datashield.assign.table}} for more details) OR a R expression with allowed assign
+#'  functions calls (see \code{\link{datashield.assign.expr}} for more details).
 #' @param variables List of variable names or Javascript expression that selects the variables of a table
 #'   (ignored if value does not refere to a table). See javascript documentation:
 #'   \url{http://opaldoc.obiba.org/en/latest/magma-user-guide/variable/}
@@ -18,24 +20,38 @@
 #'   will be the data frame row names. When specified this column can be used to perform joins between data frames.
 #' @param async Whether the result of the call should be retrieved asynchronously (TRUE means that calls are parallelized over
 #'   the connections, when the connection supports that feature, with an extra overhead of requests).
+#' @param success Callback function that will be called each time an assignment is successful. 
+#'   The expected function signature is the connection/study name. Default is NULL (no callback).
+#' @param error Callback function that will be called each time the assignment request has failed. 
+#'   The expected function signature is the connection/study name and the error message. Default is NULL (no callback).
 #'
 #' @examples
 #' \dontrun{
-#' # assign a list of variables from table HOP
-#' datashield.assign(conn, symbol="D", value="demo.HOP",
+#' # assign a list of variables from table CNSIM1
+#' datashield.assign(conn, symbol="D", value="CNSIM.CNSIM1",
 #'   variables=list("GENDER","LAB_GLUC"))
 #'
-#' # assign all the variables matching 'LAB' from table HOP
-#' datashield.assign(conn, symbol="D", value="demo.HOP",
+#' # assign all the variables matching 'LAB' from table CNSIM1
+#' datashield.assign(conn, symbol="D", value="CNSIM.CNSIM1",
 #'   variables="name().matches('LAB_')")
+#'
+#' # do assignment with callback functions
+#' datashield.assign(conns, "D", 
+#'   list(server1="CNSIM.CNSIM1", server2="CNSIM.CNSIM2"),
+#'   success = function(server) {
+#'     # do something with server's success
+#'   },
+#'   error = function(server, error) {
+#'     # do something with server's error message
+#'   })
 #' }
 #' @export
-datashield.assign <- function(conns, symbol, value, variables=NULL, missings=FALSE, identifiers=NULL, id.name=NULL, async=TRUE) {
+datashield.assign <- function(conns, symbol, value, variables=NULL, missings=FALSE, identifiers=NULL, id.name=NULL, async=TRUE, success=NULL, error=NULL) {
   .clearLastErrors()
   if(is.language(value) || is.function(value)) {
-    datashield.assign.expr(conns, symbol, value, async)
+    datashield.assign.expr(conns, symbol, value, async, success, error)
   } else {
-    datashield.assign.table(conns, symbol, value, variables, missings, identifiers, id.name, async)
+    datashield.assign.table(conns, symbol, value, variables, missings, identifiers, id.name, async, success, error)
   }
 }
 
@@ -60,15 +76,19 @@ datashield.assign <- function(conns, symbol, value, variables=NULL, missings=FAL
 #'   will be the data frame row names. When specified this column can be used to perform joins between data frames.
 #' @param async Whether the result of the call should be retrieved asynchronously. When TRUE (default) the calls are parallelized over
 #'   the connections, when the connection supports that feature, with an extra overhead of requests.
+#' @param success Callback function that will be called each time an assignment is successful. 
+#'   The expected function signature is the connection/study name. Default is NULL (no callback).
+#' @param error Callback function that will be called each time the assignment request has failed. 
+#'   The expected function signature is the connection/study name and the error message. Default is NULL (no callback).
 #'
 #' @examples
 #' \dontrun{
-#' # assign a list of variables from table HOP
-#' datashield.assign.table(conn, symbol="D", table="demo.HOP",
+#' # assign a list of variables from table CNSIM1
+#' datashield.assign.table(conn, symbol="D", table="CNSIM.CNSIM1",
 #'   variables=list("GENDER","LAB_GLUC"))
 #'
-#' # assign all the variables matching 'LAB' from table HOP
-#' datashield.assign.table(conn, symbol="D", table="demo.HOP",
+#' # assign all the variables matching 'LAB' from table CNSIM1
+#' datashield.assign.table(conn, symbol="D", table="CNSIM.CNSIM1",
 #'   variables="name().matches('LAB_')")
 #'
 #' # assign the tables that are defined in the logindata ('server' and 'table' columns are
@@ -78,10 +98,21 @@ datashield.assign <- function(conns, symbol, value, variables=NULL, missings=FAL
 #'
 #' # assign the tables that are defined in the provided named list. 
 #' # Connections are filtered by the list names.
-#' datashield.assign.table(conns, "D", list(server1="datashield.CNSIM1", server2="datashield.CNSIM2"))
+#' datashield.assign.table(conns, "D", 
+#'   list(server1="CNSIM.CNSIM1", server2="CNSIM.CNSIM2"))
+#' 
+#' # do assignment with callback functions
+#' datashield.assign.table(conns, "D", 
+#'   list(server1="CNSIM.CNSIM1", server2="CNSIM.CNSIM2"),
+#'   success = function(server) {
+#'     # do something with server's success
+#'   },
+#'   error = function(server, error) {
+#'     # do something with server's error message
+#'   })
 #' }
 #' @export
-datashield.assign.table <- function(conns, symbol, table, variables=NULL, missings=FALSE, identifiers=NULL, id.name=NULL, async=TRUE) {
+datashield.assign.table <- function(conns, symbol, table, variables=NULL, missings=FALSE, identifiers=NULL, id.name=NULL, async=TRUE, success=NULL, error=NULL) {
   .clearLastErrors()
   if (is.null(table) || length(table) == 0) {
     stop("Not a valid table name", call.=FALSE)
@@ -104,6 +135,9 @@ datashield.assign.table <- function(conns, symbol, table, variables=NULL, missin
           results[[n]] <- dsAssignTable(fconns[[n]], symbol, tables[[n]], variables, missings, identifiers, id.name, async=TRUE)
         }, error = function(e) {
           .appendError(n, e$message)
+          if (.is.callback(error)) {
+            error(n, e$message)
+          }
         })
       }
     }
@@ -115,6 +149,9 @@ datashield.assign.table <- function(conns, symbol, table, variables=NULL, missin
           results[[n]] <- dsAssignTable(fconns[[n]], symbol, tables[[n]], variables, missings, identifiers, id.name, async=FALSE)
         }, error = function(e) {
           .appendError(n, e$message)
+          if (.is.callback(error)) {
+            error(n, e$message)
+          }
         })
       }
     }
@@ -139,9 +176,15 @@ datashield.assign.table <- function(conns, symbol, table, variables=NULL, missin
                 .tickProgress(pb, tokens = list(what = paste0("Finalizing assignment ", fconns[[n]]@name, " (", symbol, " <- `", tables[[n]], "`)")))
                 dsFetch(results[[n]])
               }
+              if (completed[[n]] && .is.callback(success)) {
+                success(n)
+              }
             }, error = function(e) {
               .appendError(n, e$message)
               completed[[n]] <- TRUE
+              if (.is.callback(error)) {
+                error(n, e$message)
+              }
             })
           } else {
             completed[[n]] <- TRUE
@@ -163,8 +206,14 @@ datashield.assign.table <- function(conns, symbol, table, variables=NULL, missin
       res <- dsAssignTable(conns, symbol, tables[[conns@name]], variables, missings, identifiers, id.name)
       dsGetInfo(res)
       ignore <- dsFetch(res)
+      if (.is.callback(success)) {
+        success(conns@name)
+      }
     }, error = function(e) {
       .appendError(conns@name, e$message)
+      if (.is.callback(error)) {
+        error(conns@name, e$message)
+      }
     })
   }
   .checkLastErrors()
@@ -183,12 +232,19 @@ datashield.assign.table <- function(conns, symbol, table, variables=NULL, missin
 #'   \code{\link{datashield.login}})
 #' @param async Whether the result of the call should be retrieved asynchronously. When TRUE (default) the calls are parallelized over
 #'   the connections, when the connection supports that feature, with an extra overhead of requests.
+#' @param success Callback function that will be called each time an assignment is successful. 
+#'   The expected function signature is the connection/study name. Default is NULL (no callback).
+#' @param error Callback function that will be called each time the assignment request has failed. 
+#'   The expected function signature is the connection/study name and the error message. Default is NULL (no callback).
 #'
 #' @examples
 #' \dontrun{
-#' # assign a resource HOP
-#' datashield.assign.resource(conn, symbol="rsrc", resource="demo.HOP")
+#' # assign a resource asynchronously
+#' datashield.assign.resource(conn, symbol="rsrc", resource="RSRC.CNSIM1")
 #'
+#' # assign a resource synchronously
+#' datashield.assign.resource(conn, symbol="rsrc", resource="RSRC.CNSIM1", async = FALSE)
+#' 
 #' # assign the tables that are defined in the logindata ('server' and 'resource' columns are
 #' # expected) data frame that is used in datashield.login() function. Connections names
 #' # and server names must match.
@@ -197,10 +253,20 @@ datashield.assign.table <- function(conns, symbol, table, variables=NULL, missin
 #' # assign the resources that are defined in the provided named list. 
 #' # Connections are filtered by the list names.
 #' datashield.assign.resource(conns, "rsrc",
-#'   list(server1="datashield.CNSIM1", server2="datashield.CNSIM2"))
+#'   list(server1="RSRC.CNSIM1", server2="RSRC.CNSIM2"))
+#' 
+#' # do assignment with callback functions
+#' datashield.assign.resource(conn, symbol="rsrc",
+#'   resource = list(server1="RSRC.CNSIM1", server2="RSRC.CNSIM2"),
+#'   success = function(server) {
+#'     # do something with server's success
+#'   },
+#'   error = function(server, error) {
+#'     # do something with server's error message
+#'   })
 #' }
 #' @export
-datashield.assign.resource <- function(conns, symbol, resource, async=TRUE) {
+datashield.assign.resource <- function(conns, symbol, resource, async=TRUE, success=NULL, error=NULL) {
   .clearLastErrors()
   if (is.null(resource) || length(resource) == 0) {
     stop("Not a valid resource name", call.=FALSE)
@@ -223,6 +289,9 @@ datashield.assign.resource <- function(conns, symbol, resource, async=TRUE) {
           results[[n]] <- dsAssignResource(fconns[[n]], symbol, resources[[n]], async=TRUE)
         }, error = function(e) {
           .appendError(n, e$message)
+          if (.is.callback(error)) {
+            error(n, e$message)
+          }
         })
       }
     }
@@ -234,6 +303,9 @@ datashield.assign.resource <- function(conns, symbol, resource, async=TRUE) {
           results[[n]] <- dsAssignResource(fconns[[n]], symbol, resources[[n]], async=FALSE)
         }, error = function(e) {
           .appendError(n, e$message)
+          if (.is.callback(error)) {
+            error(n, e$message)
+          }
         })
       }
     }
@@ -258,9 +330,15 @@ datashield.assign.resource <- function(conns, symbol, resource, async=TRUE) {
                 .tickProgress(pb, tokens = list(what = paste0("Finalizing assignment ", fconns[[n]]@name, " (", symbol, " <- `", resources[[n]], "`)")))
                 dsFetch(results[[n]])
               }
+              if (completed[[n]] && .is.callback(success)) {
+                success(n)
+              }
             }, error = function(e) {
               .appendError(n, e$message)
               completed[[n]] <- TRUE
+              if (.is.callback(error)) {
+                error(n, e$message)
+              }
             })
           } else {
             completed[[n]] <- TRUE
@@ -282,8 +360,14 @@ datashield.assign.resource <- function(conns, symbol, resource, async=TRUE) {
       res <- dsAssignResource(conns, symbol, resources[[conns@name]])
       dsGetInfo(res)
       ignore <- dsFetch(res)
+      if (.is.callback(success)) {
+        success(conns@name)
+      }
     }, error = function(e) {
       .appendError(conns@name, e$message)
+      if (.is.callback(error)) {
+        error(conns@name, e$message)
+      }
     })
   }
   .checkLastErrors()
@@ -299,19 +383,35 @@ datashield.assign.resource <- function(conns, symbol, resource, async=TRUE) {
 #' @param expr R expression with allowed assign functions calls.
 #' @param async Whether the result of the call should be retrieved asynchronously. When TRUE (default) the calls are parallelized over
 #'   the connections, when the connection supports that feature, with an extra overhead of requests.
+#' @param success Callback function that will be called each time an assignment is successful. 
+#'   The expected function signature is the connection/study name. Default is NULL (no callback).
+#' @param error Callback function that will be called each time the assignment request has failed. 
+#'   The expected function signature is the connection/study name and the error message. Default is NULL (no callback).
 #'
 #' @examples
 #' \dontrun{
-#' # assign an expression to G
+#' # assign an expression to G asynchronously
 #' datashield.assign.expr(conns, symbol = "G", expr = quote(as.numeric(D$GENDER)))
 #' 
+#' # assign an expression to G synchronously
+#' datashield.assign.expr(conns, symbol = "G", expr = quote(as.numeric(D$GENDER)), async = FALSE)
+#' 
 #' # assign the expressions that are defined in the provided named list. 
 #' # Connections are filtered by the list names.
 #' datashield.assign.expr(conns, "G",
 #'   list(server1=quote(as.numeric(D$GENDER)), server2=quote(as.numeric(D$SEX))))
+#' 
+#' # do assignment with callback functions
+#' datashield.assign.expr(conns, symbol = "G", expr = quote(as.numeric(D$GENDER)),
+#'   success = function(server) {
+#'     # do something with server's success
+#'   },
+#'   error = function(server, error) {
+#'     # do something with server's error message
+#'   })
 #' }
 #' @export
-datashield.assign.expr <- function(conns, symbol, expr, async=TRUE) {
+datashield.assign.expr <- function(conns, symbol, expr, async=TRUE, success=NULL, error=NULL) {
   .clearLastErrors()
   
   # prepare expressions as a named list
@@ -331,6 +431,9 @@ datashield.assign.expr <- function(conns, symbol, expr, async=TRUE) {
           results[[n]] <- dsAssignExpr(fconns[[n]], symbol, exprs[[n]], async=TRUE)
         }, error = function(e) {
           .appendError(n, e$message)
+          if (.is.callback(error)) {
+            error(n, e$message)
+          }
         })
       }
     }
@@ -342,6 +445,9 @@ datashield.assign.expr <- function(conns, symbol, expr, async=TRUE) {
           results[[n]] <- dsAssignExpr(fconns[[n]], symbol, exprs[[n]], async=FALSE)
         }, error = function(e) {
           .appendError(n, e$message)
+          if (.is.callback(error)) {
+            error(n, e$message)
+          }
         })
       }
     }
@@ -367,9 +473,15 @@ datashield.assign.expr <- function(conns, symbol, expr, async=TRUE) {
                 .tickProgress(pb, tokens = list(what = paste0("Finalizing assignment ", fconns[[n]]@name, " (", symbol, " <- ", dexpr, ")")))
                 dsFetch(results[[n]])
               }
+              if (completed[[n]] && .is.callback(success)) {
+                success(n)
+              }
             }, error = function(e) {
               .appendError(n, e$message)
               completed[[n]] <- TRUE
+              if (.is.callback(error)) {
+                error(n, e$message)
+              }
             })
           } else {
             completed[[n]] <- TRUE
@@ -390,8 +502,14 @@ datashield.assign.expr <- function(conns, symbol, expr, async=TRUE) {
     tryCatch({
       res <- dsAssignExpr(conns, symbol, exprs[[conns@name]])
       ignore <- dsFetch(res)
+      if (.is.callback(success)) {
+        success(conns@name)
+      }
     }, error = function(e) {
       .appendError(conns@name, e$message)
+      if (.is.callback(error)) {
+        error(conns@name, e$message)
+      }
     })
   }
   .checkLastErrors()
